@@ -2,8 +2,8 @@
    OpenCoarrays is distributed under the OSI-approved BSD 3-clause License:
    OpenCoarrays -- ISO_Fortran_binding standard-compliant interoperability with
    C.
-   Copyright (c) 2015, Sourcery, Inc.
-   Copyright (c) 2015, Sourcery Institute
+   Copyright (c) 2018, Sourcery, Inc.
+   Copyright (c) 2018, Sourcery Institute
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -383,17 +383,17 @@ int CFI_is_contiguous (const CFI_cdesc_t *dv)
   for (int i = 0; i < dv->rank; i++)
     {
       if (dv->dim[i].sm != dv->elem_len)
-        return CFI_FAILURE;
+        return 0;
     }
 
   /* Allocatable arrays are always contiguous. */
   if (dv->attribute == CFI_attribute_allocatable)
     {
-      return CFI_SUCCESS;
+      return 1;
     }
   else
     {
-      return CFI_FAILURE;
+      return 0;
     }
 }
 
