@@ -2,8 +2,8 @@
    OpenCoarrays is distributed under the OSI-approved BSD 3-clause License:
    OpenCoarrays -- ISO_Fortran_binding standard-compliant interoperability with
    C.
-   Copyright (c) 2015, Sourcery, Inc.
-   Copyright (c) 2015, Sourcery Institute
+   Copyright (c) 2018, Sourcery, Inc.
+   Copyright (c) 2018, Sourcery Institute
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@ int main (void)
   int    ind;
   size_t base_type;
   size_t base_type_size;
-  size_t errno;
+  size_t iso_errno;
 
   /* Test function establish. */
   /* Fresh descriptor, base address is NULL. */
@@ -77,7 +77,7 @@ int main (void)
           /* Loop through rank. */
           for (int k = 0; k <= CFI_MAX_RANK; k++)
             {
-              errno = 1;
+              iso_errno = 1;
               rank  = k;
               CFI_CDESC_T (rank) test1;
               /* We do this because C sometimes doesn't make the structures with
@@ -154,7 +154,7 @@ int main (void)
           /* Loop through rank. */
           for (int k = 0; k <= CFI_MAX_RANK; k++)
             {
-              errno = 1;
+              iso_errno = 1;
               rank  = k;
               if (extents != NULL)
                 {
@@ -284,7 +284,7 @@ int main (void)
           /* Loop through rank. */
           for (int k = 0; k <= CFI_MAX_RANK; k++)
             {
-              errno = 1;
+              iso_errno = 1;
               rank  = k;
               if (extents != NULL)
                 {
@@ -358,7 +358,7 @@ int main (void)
     }
 
   rank  = 1;
-  errno = 1;
+  iso_errno = 1;
   CFI_CDESC_T (rank) test4;
   base_type      = type[3] & CFI_type_mask;
   base_type_size = (type[3] - base_type) >> CFI_type_kind_shift;
@@ -373,7 +373,7 @@ int main (void)
     }
 
   rank  = 1;
-  errno = 1;
+  iso_errno = 1;
   CFI_CDESC_T (rank) test5;
   base_type      = type[3] & CFI_type_mask;
   base_type_size = (type[3] - base_type) >> CFI_type_kind_shift;
@@ -389,7 +389,7 @@ int main (void)
 
   /* Test CFI_deallocate. */
   rank           = 1;
-  errno          = 1;
+  iso_errno          = 1;
   base_type      = type[3] & CFI_type_mask;
   base_type_size = (type[3] - base_type) >> CFI_type_kind_shift;
   for (int i = 1; i <= 3; i++)
@@ -414,11 +414,14 @@ int main (void)
       ind = CFI_establish ((CFI_cdesc_t *) &test6, NULL, attribute, type[i],
                            elem_len, rank, extents);
       ind = CFI_allocate ((CFI_cdesc_t *) &test6, lower, upper, base_type_size);
-      ind = CFI_deallocate ((CFI_cdesc_t *) &test6);
-      if (ind != CFI_INVALID_ATTRIBUTE && test6.base_addr != NULL)
+      if (ind == CFI_SUCCESS)
         {
-          printf ("CFI_deallocate: failed to deallocate memory.\n");
-          return 1;
+          ind = CFI_deallocate ((CFI_cdesc_t *) &test6);
+          if (ind != CFI_INVALID_ATTRIBUTE && test6.base_addr != NULL)
+            {
+              printf ("CFI_deallocate: failed to deallocate memory.\n");
+              return 1;
+            }
         }
     }
 
@@ -431,7 +434,7 @@ int main (void)
       attribute = i;
       for (int j = 0; j <= 4; j++)
         {
-          errno = 1;
+          iso_errno = 1;
           rank  = j;
           if (extents != NULL)
             {
@@ -459,6 +462,10 @@ int main (void)
                                elem_len, rank, extents);
           tmp_ind = CFI_allocate ((CFI_cdesc_t *) &test7, lower, upper,
                                   base_type_size);
+          if (tmp_ind != CFI_SUCCESS)
+            {
+              goto next_attribute4;
+            }
           ind = CFI_is_contiguous ((CFI_cdesc_t *) &test7);
           if (ind != CFI_INVALID_RANK && rank == 0 &&
               tmp_ind != CFI_INVALID_ATTRIBUTE)
@@ -474,6 +481,7 @@ int main (void)
               return 1;
             }
         }
+      next_attribute4:;
     }
 
   /* Test CFI_address. */
@@ -521,7 +529,7 @@ int main (void)
           /* Loop through rank. */
           for (int k = 1; k <= CFI_MAX_RANK; k++)
             {
-              errno = 1;
+              iso_errno = 1;
               rank  = k;
               CFI_CDESC_T (rank) source;
               if (extents != NULL)
@@ -608,7 +616,7 @@ int main (void)
   for (int i = 0; i < CFI_MAX_RANK; i++)
     {
       rank           = i;
-      errno          = 1;
+      iso_errno          = 1;
       base_type      = type[3] & CFI_type_mask;
       base_type_size = (type[3] - base_type) >> CFI_type_kind_shift;
       attribute      = CFI_attribute_other;
@@ -635,7 +643,7 @@ int main (void)
         {
           extents[r] = r + 2;
         }
-      ind = CFI_establish ((CFI_cdesc_t *) &test8b, &errno, attribute, type[3],
+      ind = CFI_establish ((CFI_cdesc_t *) &test8b, &iso_errno, attribute, type[3],
                            base_type_size, rank, extents);
       ind = CFI_setpointer ((CFI_cdesc_t *) &test8a, (CFI_cdesc_t *) &test8b,
                             lower);
@@ -676,7 +684,7 @@ int main (void)
 
   /* NULL source. */
   rank           = 10;
-  errno          = 1;
+  iso_errno          = 1;
   base_type      = type[3] & CFI_type_mask;
   base_type_size = (type[3] - base_type) >> CFI_type_kind_shift;
   CFI_CDESC_T (rank) test9;
@@ -711,7 +719,7 @@ int main (void)
     }
 
   rank      = 3;
-  errno     = 1;
+  iso_errno     = 1;
   attribute = CFI_attribute_other;
   CFI_CDESC_T (rank) test10a, test10b;
   if (extents != NULL)
@@ -739,7 +747,7 @@ int main (void)
     }
   base_type      = CFI_type_double & CFI_type_mask;
   base_type_size = (CFI_type_double - base_type) >> CFI_type_kind_shift;
-  ind            = CFI_establish ((CFI_cdesc_t *) &test10b, &errno, attribute,
+  ind            = CFI_establish ((CFI_cdesc_t *) &test10b, &iso_errno, attribute,
                        CFI_type_double, base_type_size, rank, extents);
   ind = CFI_setpointer ((CFI_cdesc_t *) &test10a, (CFI_cdesc_t *) &test10b,
                         lower);
@@ -749,14 +757,14 @@ int main (void)
       return 1;
     }
 
-  errno          = 1;
+  iso_errno          = 1;
   base_type      = CFI_type_other & CFI_type_mask;
   base_type_size = 666;
   ind            = CFI_establish ((CFI_cdesc_t *) &test10a, &ind, attribute,
                        CFI_type_other, base_type_size, rank, extents);
   base_type      = CFI_type_other & CFI_type_mask;
   base_type_size = 69;
-  ind            = CFI_establish ((CFI_cdesc_t *) &test10b, &errno, attribute,
+  ind            = CFI_establish ((CFI_cdesc_t *) &test10b, &iso_errno, attribute,
                        CFI_type_other, base_type_size, rank, extents);
   ind = CFI_setpointer ((CFI_cdesc_t *) &test10a, (CFI_cdesc_t *) &test10b,
                         lower);
@@ -766,7 +774,7 @@ int main (void)
       return 1;
     }
 
-  errno          = 1;
+  iso_errno          = 1;
   base_type      = type[3] & CFI_type_mask;
   base_type_size = (CFI_type_long - base_type) >> CFI_type_kind_shift;
   ind = CFI_establish ((CFI_cdesc_t *) &test10a, &ind, attribute, type[3],
@@ -790,7 +798,7 @@ int main (void)
     }
   base_type      = CFI_type_other & CFI_type_mask;
   base_type_size = (CFI_type_long - base_type) >> CFI_type_kind_shift;
-  ind = CFI_establish ((CFI_cdesc_t *) &test10c, &errno, attribute, type[3],
+  ind = CFI_establish ((CFI_cdesc_t *) &test10c, &iso_errno, attribute, type[3],
                        base_type_size, rank, extents);
   ind = CFI_setpointer ((CFI_cdesc_t *) &test10a, (CFI_cdesc_t *) &test10c,
                         lower);
@@ -839,7 +847,7 @@ int main (void)
       /* Loop through rank. */
       for (int k = 1; k <= CFI_MAX_RANK; k++)
         {
-          errno = 1;
+          iso_errno = 1;
           rank  = k;
           CFI_CDESC_T (rank) section, source;
           if (extents != NULL)
@@ -875,7 +883,7 @@ int main (void)
                                CFI_attribute_other, type[i], elem_len, rank,
                                NULL);
           ind = CFI_allocate ((CFI_cdesc_t *) &source, lower, upper, elem_len);
-          if (ind == CFI_ERROR_MEM_ALLOCATION)
+          if (ind != CFI_SUCCESS)
             {
               goto next_type2;
             }
@@ -1002,7 +1010,7 @@ int main (void)
     next_type2:;
     }
 
-  errno = 1;
+  iso_errno = 1;
   rank  = 1;
   CFI_CDESC_T (rank) section, source;
   if (extents != NULL)
@@ -1036,48 +1044,52 @@ int main (void)
   ind = CFI_establish ((CFI_cdesc_t *) &section, NULL, CFI_attribute_other,
                        type[3], elem_len, rank, NULL);
   ind = CFI_allocate ((CFI_cdesc_t *) &source, lower, upper, elem_len);
-  for (int r = 0; r < rank; r++)
+  if (ind == CFI_SUCCESS)
     {
-      lower[r]   = rank - r - 3;
-      strides[r] = r + 1;
-      upper[r]   = lower[r] + extents[r] - 3;
-    }
-  ind = CFI_section ((CFI_cdesc_t *) &section, NULL, lower, upper, strides);
-  if (ind != CFI_INVALID_DESCRIPTOR)
-    {
-      printf ("CFI_section: failed to detect that source is NULL.\n");
-      return 1;
-    }
-  ind = CFI_section (NULL, (CFI_cdesc_t *) &source, lower, upper, strides);
-  if (ind != CFI_INVALID_DESCRIPTOR)
-    {
-      printf ("CFI_section: failed to detect that section is NULL.\n");
-      return 1;
-    }
-  ind =
-      CFI_establish ((CFI_cdesc_t *) &section, NULL, CFI_attribute_allocatable,
-                     type[3], elem_len, rank, NULL);
-  ind = CFI_section ((CFI_cdesc_t *) &section, (CFI_cdesc_t *) &source, lower,
-                     upper, strides);
-  if (ind != CFI_INVALID_ATTRIBUTE)
-    {
-      printf ("CFI_section: failed to detect invalid attribute.\n");
-      return 1;
-    }
-  ind = CFI_establish ((CFI_cdesc_t *) &section, NULL, CFI_attribute_other,
-                       type[3], elem_len, rank, NULL);
-  ind = CFI_deallocate ((CFI_cdesc_t *) &source);
-  ind = CFI_section ((CFI_cdesc_t *) &section, (CFI_cdesc_t *) &source, lower,
-                     upper, strides);
-  if (ind != CFI_ERROR_BASE_ADDR_NULL)
-    {
-      printf ("CFI_section: failed to detect that the base address is NULL.\n");
-      return 1;
+      for (int r = 0; r < rank; r++)
+        {
+          lower[r]   = rank - r - 3;
+          strides[r] = r + 1;
+          upper[r]   = lower[r] + extents[r] - 3;
+        }
+      ind = CFI_section ((CFI_cdesc_t *) &section, NULL, lower, upper, strides);
+      if (ind != CFI_INVALID_DESCRIPTOR)
+        {
+          printf ("CFI_section: failed to detect that source is NULL.\n");
+          return 1;
+        }
+      ind = CFI_section (NULL, (CFI_cdesc_t *) &source, lower, upper, strides);
+      if (ind != CFI_INVALID_DESCRIPTOR)
+        {
+          printf ("CFI_section: failed to detect that section is NULL.\n");
+          return 1;
+        }
+      ind =
+          CFI_establish ((CFI_cdesc_t *) &section, NULL, CFI_attribute_allocatable,
+                         type[3], elem_len, rank, NULL);
+      ind = CFI_section ((CFI_cdesc_t *) &section, (CFI_cdesc_t *) &source, lower,
+                         upper, strides);
+      if (ind != CFI_INVALID_ATTRIBUTE)
+        {
+          printf ("CFI_section: failed to detect invalid attribute.\n");
+          return 1;
+        }
+      ind = CFI_establish ((CFI_cdesc_t *) &section, NULL, CFI_attribute_other,
+                           type[3], elem_len, rank, NULL);
+      ind = CFI_deallocate ((CFI_cdesc_t *) &source);
+      ind = CFI_section ((CFI_cdesc_t *) &section, (CFI_cdesc_t *) &source, lower,
+                         upper, strides);
+      if (ind != CFI_ERROR_BASE_ADDR_NULL)
+        {
+          printf ("CFI_section: failed to detect that the base address is NULL.\n");
+          return 1;
+        }
     }
+
   CFI_CDESC_T (0) section2, source2;
   ind = CFI_establish ((CFI_cdesc_t *) &source2, &ind, CFI_attribute_other,
                        type[3], 0, 0, NULL);
-  ind = CFI_establish ((CFI_cdesc_t *) &section2, &errno, CFI_attribute_other,
+  ind = CFI_establish ((CFI_cdesc_t *) &section2, &iso_errno, CFI_attribute_other,
                        type[3], 0, 0, NULL);
   ind = CFI_section ((CFI_cdesc_t *) &section2, (CFI_cdesc_t *) &source2, lower,
                      upper, strides);
@@ -1086,38 +1098,42 @@ int main (void)
       printf ("CFI_section: failed to detect invalid rank.\n");
       return 1;
     }
+
   ind = CFI_establish ((CFI_cdesc_t *) &source, NULL, CFI_attribute_allocatable,
                        type[3], 0, rank, extents);
   ind = CFI_establish ((CFI_cdesc_t *) &section, NULL, CFI_attribute_other,
                        type[6], 0, rank, NULL);
   ind = CFI_allocate ((CFI_cdesc_t *) &source, lower, upper, elem_len);
-  for (int r = 0; r < rank; r++)
+  if (ind == CFI_SUCCESS)
     {
-      lower[r]   = rank - r - 3;
-      strides[r] = r + 1;
-      upper[r]   = lower[r] + extents[r] - 3;
-    }
-  ind = CFI_section ((CFI_cdesc_t *) &section, (CFI_cdesc_t *) &source, lower,
-                     upper, strides);
-  if (ind != CFI_INVALID_ELEM_LEN)
-    {
-      printf ("CFI_section: failed to detect incompatible element lengths "
-              "between source and section.\n");
-      return 1;
-    }
-  ind = CFI_establish ((CFI_cdesc_t *) &section, NULL, CFI_attribute_other,
-                       CFI_type_long, 0, rank, NULL);
-  ind = CFI_section ((CFI_cdesc_t *) &section, (CFI_cdesc_t *) &source, lower,
-                     upper, strides);
-  if (ind != CFI_INVALID_TYPE)
-    {
-      printf ("CFI_section: failed to detect invalid type.\n");
-      return 1;
+      for (int r = 0; r < rank; r++)
+        {
+          lower[r]   = rank - r - 3;
+          strides[r] = r + 1;
+          upper[r]   = lower[r] + extents[r] - 3;
+        }
+      ind = CFI_section ((CFI_cdesc_t *) &section, (CFI_cdesc_t *) &source, lower,
+                         upper, strides);
+      if (ind != CFI_INVALID_ELEM_LEN)
+        {
+          printf ("CFI_section: failed to detect incompatible element lengths "
+                  "between source and section.\n");
+          return 1;
+        }
+      ind = CFI_establish ((CFI_cdesc_t *) &section, NULL, CFI_attribute_other,
+                           CFI_type_long, 0, rank, NULL);
+      ind = CFI_section ((CFI_cdesc_t *) &section, (CFI_cdesc_t *) &source, lower,
+                         upper, strides);
+      if (ind != CFI_INVALID_TYPE)
+        {
+          printf ("CFI_section: failed to detect invalid type.\n");
+          return 1;
+        }
     }
 
   for (int i = 1; i < CFI_MAX_RANK; i++)
     {
-      errno   = 1;
+      iso_errno   = 1;
       rank    = i;
       int ctr = 0;
       CFI_CDESC_T (rank) source;
@@ -1151,7 +1167,7 @@ int main (void)
           CFI_establish ((CFI_cdesc_t *) &source, NULL,
                          CFI_attribute_allocatable, type[3], 0, rank, extents);
       ind = CFI_allocate ((CFI_cdesc_t *) &source, lower, upper, elem_len);
-      if (ind == CFI_ERROR_MEM_ALLOCATION)
+      if (ind != CFI_SUCCESS)
         {
           continue;
         }
@@ -1217,7 +1233,7 @@ int main (void)
     }
 
   /* CFI_section negative strides. */
-  errno = 1;
+  iso_errno = 1;
   rank  = 8;
   if (extents != NULL)
     {
@@ -1252,13 +1268,15 @@ int main (void)
   ind = CFI_establish ((CFI_cdesc_t *) &section3, NULL, CFI_attribute_other,
                        type[3], 0, rank, NULL);
   ind = CFI_allocate ((CFI_cdesc_t *) &source3, lower, upper, elem_len);
-
-  ind = CFI_section ((CFI_cdesc_t *) &section3, (CFI_cdesc_t *) &source3, upper,
-                     lower, strides);
-  if (ind != CFI_SUCCESS && ind != CFI_INVALID_STRIDE)
+  if (ind == CFI_SUCCESS)
     {
-      printf ("CFI_section: failed to detect invalid stride.\n");
-      return 1;
+      ind = CFI_section ((CFI_cdesc_t *) &section3, (CFI_cdesc_t *) &source3,
+                         upper, lower, strides);
+      if (ind != CFI_SUCCESS && ind != CFI_INVALID_STRIDE)
+        {
+          printf ("CFI_section: failed to detect invalid stride.\n");
+          return 1;
+        }
     }
 
   /* CFI_select_part */
